@@ -4,9 +4,9 @@ Copyright (C) 1994, 1996, 1999, 2000
 
 Author: Hironobu Takahashi, Masahiko Sato, Kiyotaka Sakai, Kenji Yabuuchi
 Maintainer: Mikio Nakajima <minakaji@osaka.email.ne.jp>
-Version: $Id: skkdic-expr.c,v 1.10 2006/01/04 10:35:06 skk-cvs Exp $
+Version: $Id: skkdic-expr.c,v 1.20 2007/09/20 09:46:41 skk-cvs Exp $
 Keywords: japanese
-Last Modified: $Date: 2006/01/04 10:35:06 $
+Last Modified: $Date: 2007/09/20 09:46:41 $
 
 This file is part of Daredevil SKK.
 
@@ -31,6 +31,13 @@ Boston, MA 02110-1301, USA. */
 
 #include <config.h>
 #include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
+#ifdef HAVE_MKDTEMP
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
+#endif /* HAVE_UNISTD_H */
+#endif /* HAVE_MKDTEMP */
 
 #ifdef HAVE_LIBDB
 #define DB_DBM_HSEARCH 1
@@ -92,6 +99,7 @@ extern int errno;
 char file_name[256];
 char okuri_tail_name[256];
 char okuri_head_name[256];
+char tmpsubdir[256];
 
 /* ѥǡ١ */
 DBM *db;
@@ -99,6 +107,22 @@ DBM *okuriheaddb;
 DBM *okuritaildb;
 FILE *dbcontent;
 
+/* snprintf() ʤĶΤ */
+#ifndef HAVE_SNPRINTF
+/* #include <stdio.h> */
+#include <stdarg.h>
+int snprintf(char *s, size_t maxlen, const char *format, ...)
+{
+    va_list ap;
+    int n;
+
+    va_start(ap, format);
+    n = vsprintf(s, format, ap);
+    va_end(ap);
+    return n;
+}
+#endif
+
 /* ꤬ʤĤȥޤƽԤ碌뤫ɤ */
 int	okurigana_flag;
 
@@ -127,28 +151,28 @@ static void db_remove_files()
     char dir_name[256];
 
     db_remove_file(file_name);
-    sprintf(pag_name, "%s.pag", file_name);
+    snprintf(pag_name, sizeof(pag_name), "%s.pag", file_name);
     db_remove_file(pag_name);
-    sprintf(dir_name, "%s.dir", file_name);
+    snprintf(dir_name, sizeof(dir_name), "%s.dir", file_name);
     db_remove_file(dir_name);
-    sprintf(dir_name, "%s.db", file_name);
+    snprintf(dir_name, sizeof(dir_name), "%s.db", file_name);
     db_remove_file(dir_name);
 
     if (okurigana_flag) {
 	db_remove_file(okuri_head_name);
-	sprintf(pag_name, "%s.pag", okuri_head_name);
+	snprintf(pag_name, sizeof(pag_name), "%s.pag", okuri_head_name);
 	db_remove_file(pag_name);
-	sprintf(dir_name, "%s.dir", okuri_head_name);
+	snprintf(dir_name, sizeof(dir_name), "%s.dir", okuri_head_name);
 	db_remove_file(dir_name);
-	sprintf(dir_name, "%s.db", okuri_head_name);
+	snprintf(dir_name, sizeof(dir_name), "%s.db", okuri_head_name);
 	db_remove_file(dir_name);
 
 	db_remove_file(okuri_tail_name);
-	sprintf(pag_name, "%s.pag", okuri_tail_name);
+	snprintf(pag_name, sizeof(pag_name), "%s.pag", okuri_tail_name);
 	db_remove_file(pag_name);
-	sprintf(dir_name, "%s.dir", okuri_tail_name);
+	snprintf(dir_name, sizeof(dir_name), "%s.dir", okuri_tail_name);
 	db_remove_file(dir_name);
-	sprintf(dir_name, "%s.db", okuri_tail_name);
+	snprintf(dir_name, sizeof(dir_name), "%s.db", okuri_tail_name);
 	db_remove_file(dir_name);
     }
 }
@@ -235,6 +259,7 @@ signal_handler (signo)
      int signo;
 {
     db_remove_files();
+    rmdir(tmpsubdir);
     signal(signo, SIG_DFL);
     kill(getpid(), signo);
 }
@@ -786,7 +811,11 @@ int main(argc, argv)
     char *tmpdir;
 
     output = stdout;
-    tmpdir = TMPDIR;
+
+    tmpdir = getenv("TMPDIR");
+    if (tmpdir == NULL) {
+      tmpdir = TMPDIR;
+    }
 
     /* ν */
     for (i = 1; i < argc; ++ i) {
@@ -816,14 +845,25 @@ int main(argc, argv)
 	exit(1);
     }
 
-    sprintf(file_name, "%s/skkdic%d", tmpdir, getpid());
+#ifdef HAVE_MKDTEMP
+    snprintf(tmpsubdir, sizeof(tmpsubdir), "%s/skkdicXXXXXX", tmpdir);
+    if (mkdtemp(tmpsubdir) == NULL) {
+#else /* not HAVE_MKDTEMP */
+    snprintf(tmpsubdir, sizeof(tmpsubdir), "%s/skkdic%d", tmpdir, getpid());
+    if (mkdir(tmpsubdir, 0700)) {
+#endif /* not HAVE_MKDTEMP */
+	perror(tmpsubdir);
+	exit(1);
+    }
+    tmpdir = tmpsubdir;
+    snprintf(file_name, sizeof(file_name), "%s/skkdic%d", tmpdir, getpid());
     if (okurigana_flag) {
-	sprintf(okuri_head_name, "%s/skkhead%d", tmpdir, getpid());
-	sprintf(okuri_tail_name, "%s/skktail%d", tmpdir, getpid());
+	snprintf(okuri_head_name, sizeof(okuri_head_name), "%s/skkhead%d", tmpdir, getpid());
+	snprintf(okuri_tail_name, sizeof(okuri_tail_name), "%s/skktail%d", tmpdir, getpid());
     }
     set_signal_handler();
     db_make_files();
-    
+
     negate = 0;
     for (; i < argc; ++ i) {
 	if (argv[i][0] == '+') {
@@ -845,6 +885,7 @@ int main(argc, argv)
     }
     type_out(output);
     db_remove_files();
+    rmdir(tmpsubdir);
     return 0;
 }
 
